@@ -175,45 +175,47 @@ class PromptManager:
 
             return get_token_distributions
         token_distributions=[]
-        for i,chunk_prompt in enumerate(prompt_data):
-            tokenize_js={"notation":notation,
-                "max_tokens":max_tokens,
-                "completion_percentage":completion_percentage,
-                "prompt_guide":self.create_prompt_guide(request=request[i],chunk_token_distribution_number=i),
-                "chunk_prompt":chunk_prompt}
-            total_prompt = ''
-            for each in tokenize_js.keys():
-                if each != "chunk_prompt":
-                    total_prompt+=str(tokenize_js[each])
-                    
-            max_tokens = int(tokenize_js["max_tokens"])
-            
-            completion_percent = convert_to_percentage(tokenize_js["completion_percentage"])
-            completion_token_desired = int(max_tokens*completion_percent)
-            
-            
-            prompt_percent = convert_to_percentage(100-tokenize_js["completion_percentage"])
-            prompt_token_desired = int(max_tokens*prompt_percent)
-            bot_notation_token_count=int(200)
-            initial_prompt_token_length = num_tokens_from_string(str(total_prompt))+bot_notation_token_count
+        try:
+            for i,chunk_prompt in enumerate(prompt_data):
+                tokenize_js={"notation":notation,
+                    "max_tokens":max_tokens,
+                    "completion_percentage":completion_percentage,
+                    "prompt_guide":self.create_prompt_guide(request=request[i],chunk_token_distribution_number=i),
+                    "chunk_prompt":chunk_prompt}
+                total_prompt = ''
+                for each in tokenize_js.keys():
+                    if each != "chunk_prompt":
+                        total_prompt+=str(tokenize_js[each])
+                        
+                max_tokens = int(tokenize_js["max_tokens"])
+                
+                completion_percent = convert_to_percentage(tokenize_js["completion_percentage"])
+                completion_token_desired = int(max_tokens*completion_percent)
+                
+                
+                prompt_percent = convert_to_percentage(100-tokenize_js["completion_percentage"])
+                prompt_token_desired = int(max_tokens*prompt_percent)
+                bot_notation_token_count=int(200)
+                initial_prompt_token_length = num_tokens_from_string(str(total_prompt))+bot_notation_token_count
 
-            initial_completion_token_length=bot_notation_token_count
+                initial_completion_token_length=bot_notation_token_count
+                
+                total_chunk_data = tokenize_js["chunk_prompt"] or ''
             
-            total_chunk_data = tokenize_js["chunk_prompt"] or ''
-        
-            total_chunk_token_length = num_tokens_from_string(str(total_chunk_data))
-            ficticious_chunk_token_length = prompt_token_desired-initial_prompt_token_length
-            num_chunks = total_chunk_token_length // ficticious_chunk_token_length      
-            instruction_token_size = 0
-            
-            token_distributions.append(get_token_distributions(prompt_token_desired=prompt_token_desired,
-                                                         initial_prompt_token_length=initial_prompt_token_length,
-                                                         total_chunk_data=total_chunk_data,
-                                                         fictitious_chunk_token_length=ficticious_chunk_token_length,
-                                                         completion_token_desired=completion_token_desired,
-                                                         initial_completion_token_length=initial_completion_token_length,
-                                                         chunk_type=self.chunk_type))
-       
+                total_chunk_token_length = num_tokens_from_string(str(total_chunk_data))
+                ficticious_chunk_token_length = prompt_token_desired-initial_prompt_token_length
+                num_chunks = total_chunk_token_length // ficticious_chunk_token_length      
+                instruction_token_size = 0
+                
+                token_distributions.append(get_token_distributions(prompt_token_desired=prompt_token_desired,
+                                                             initial_prompt_token_length=initial_prompt_token_length,
+                                                             total_chunk_data=total_chunk_data,
+                                                             fictitious_chunk_token_length=ficticious_chunk_token_length,
+                                                             completion_token_desired=completion_token_desired,
+                                                             initial_completion_token_length=initial_completion_token_length,
+                                                             chunk_type=self.chunk_type))
+        except:
+            pass
         return token_distributions
             
     def create_prompt_guide(self,
