@@ -1,5 +1,6 @@
 import os
 import requests
+import json
 from abstract_utilities import (get_date,
                                 mkdirs,
                                 get_files,
@@ -10,14 +11,15 @@ from abstract_utilities import (get_date,
                                 get_highest_value_obj,
                                 get_any_value,
                                 safe_json_loads,
-                                safe_dump_to_file)
+                                safe_dump_to_file,
+                                read_from_file,
+                                write_to_file)
 class SaveManager:
     """
     Manages the saving of data. This class should provide methods to specify where (e.g., what database or file) and how (e.g., in what format) data should be saved.
     """
-    def __init__(self, data,title=None,directory=None,model='default'):
+    def __init__(self, data={},title=None,directory=None,model='default'):
         self.title=title
-        self.data = safe_json_loads(data)
         self.model=model
         self.date = get_date()
         self.directory = mkdirs(directory or os.path.join(os.getcwd(), 'response_data'))
@@ -25,7 +27,12 @@ class SaveManager:
         self.directory = mkdirs(os.path.join(self.directory, self.model))
         self.file_name = self.create_unique_file_name()
         self.file_path = os.path.join(self.directory, self.file_name)
-        safe_dump_to_file(data=data,file_path=self.file_path)
+        if data:
+            self.data = safe_json_loads(data)
+            self.data['file_path']=self.file_path
+            self.data['title']=self.title
+            self.data['model']=self.model
+            self.save_to_file(data = data,file_path = self.file_path)
     def create_unique_file_name(self):
         base_name = f"{self.title}.json"
         index = 1
@@ -33,7 +40,16 @@ class SaveManager:
         while os.path.exists(os.path.join(self.directory, unique_name)):
             unique_name = f"{self.title}_{index}.json"
             index += 1
-        return unique_name       
+        return unique_name
+    def save_to_file(self, data, file_path):
+        # Assuming `data` is already a dictionary, we convert it to a JSON string and save.
+        with open(file_path, 'w', encoding='utf-8') as file:
+            json.dump(data, file, ensure_ascii=False, indent=4)
+    
+    @staticmethod
+    def read_saved_json(file_path):
+        # Use 'safe_read_from_json' which is presumed to handle errors and return JSON
+        return safe_read_from_json(file_path)
 class ResponseManager:
     """
     The `ResponseManager` class handles the communication process with AI models by managing the sending of queries and storage of responses. It ensures that responses are correctly interpreted, errors are managed, and that responses are saved in a structured way, facilitating easy retrieval and analysis.
@@ -70,13 +86,13 @@ class ResponseManager:
         initial_query: Manages the initial sequence of sending queries and processing responses.
 
     """
-    def __init__(self,prompt_mgr,api_mgr,title=None,directory=None):
+    def __init__(self,prompt_mgr=None,api_mgr=None,title=None,directory=None):
         self.prompt_mgr=prompt_mgr
         self.model_mgr=self.prompt_mgr.model_mgr
         self.api_mgr=api_mgr
         self.title=title
         self.directory=mkdirs(directory or os.path.join(os.getcwd(),'response_data'))
-        self.token_dist=prompt_mgr.token_dist
+        self.chunk_token_distributions=prompt_mgr.chunk_token_distributions
         self.output=[]
         self.content={}
         self.query_js={}
@@ -84,13 +100,12 @@ class ResponseManager:
         self.chunk_descriptions=[]
         self.original_title=self.title
         self.re_initialize_query()
+        self.response_keys = ["api_response","abort","additional_responses","suggestions","notation","generate_title","request_chunks","token_adjustment","prompt_as_previous","created","model","error",'content']
     def re_initialize_query(self):
         self.i_query=0
         self.generate_title=None
         self.query_done=False
         self.abort_js = {"abort":None,"additional_responses":None,"request_chunks":None}
-        self.response_keys = ["api_response","abort","additional_responses","suggestions","notation","generate_title","request_chunks","token_adjustment","prompt_as_previous"]
-        self.original_title=None
         self.suggestions=False
         self.notation=False
         self.api_response={}
@@ -118,17 +133,18 @@ class ResponseManager:
             raise Exception(f'Request failed with status code {self.api_response.status_code}\n{self.api_response.text}\n\n')
         return self.get_response()
     def get_response(self):
-        """
-        Extracts and returns the response dictionary from the API response.
-
-        Returns:
-            dict: Extracted response dictionary.
-        """
+        # Convert the response to JSON directly within this method
         try:
             self.api_response = self.response.json()
-        except:
-            self.api_response = self.response.text
-        return self.api_response
+        except ValueError:
+            # Handle the case where the response is not JSON
+            self.api_response = {"error": "Response is not in JSON format", "data": self.response.text}
+        # Save the response using SaveManager
+        json_api_response = safe_json_loads(self.api_response)
+        self.get_all_instruction(json_api_response)
+        self.query_js["response"]=json_api_response
+        self.save_manager = SaveManager(data=self.query_js, title=self.generate_title or self.title or 'response', directory=self.directory, model=self.model)
+        self.output.append(self.save_manager.data)
     def try_load_response(self):
         """
         Attempts to load the response content into a structured format.
@@ -139,21 +155,9 @@ class ResponseManager:
         Processes the response and manages the creation of a save point through `SaveManager`.
         """
         return self.query_js
-    def get_last_response(self):
-        """
-        Retrieves the last response from the save location.
-
-        Returns:
-            tuple: A tuple containing the file path and the response dictionary.
-        """
-        self.recent_file = get_highest_value_obj(get_files(self.directory),function=get_file_create_time)
-        self.last_response = safe_json_loads(safe_read_from_json(self.recent_file))
-        self.content = safe_json_loads(get_any_value(self.api_response,'content'))
-        self.response_js= safe_json_loads(get_any_value(self.content, 'api_response'))
-        return self.recent_file,self.response_js
-    def get_abort():
+    def get_abort(self):
        self.abort_js = {"abort":None,"additional_responses":None,"request_chunks":None}
-       for key,value in self.prompt_mgr.instructions_js.items():
+       for key,value in self.prompt_mgr.instruction_mgr.instructions_js.items():
            if key in list(self.abort_js.keys()):
               setattr(self,key,value)
               self.abort_js[key]=value
@@ -163,41 +167,40 @@ class ResponseManager:
            return False
        else:
            return True
+    def get_all_instruction(self,data):
+        self.prompt_as_previous_clone
+        if data:
+            for i,key in enumerate(self.response_keys):
+                value = get_any_value(data,key)
+                if isinstance(value,list) and value:
+                    value=value[-1]
+                setattr(self,key,value)
+                if key == 'prompt_as_previous' and value:
+                    self.prompt_as_previous_clone=self.api_response
+            error_response = get_any_value(data,'error')
+            if error_response:
+                self.error = 'error'
+            self.model = self.model or self.error or 'default'
+            self.title = self.generate_title or self.original_title or self.created
+            self.get_abort()
     def send_query(self):
         """
         Handles the response after a query has been sent.
         """
-        self.prompt = self.prompt_mgr.create_prompt(dist_number=self.i_query,
-                                                    bot_notation=self.notation,
+        self.query_js={}
+       
+        self.prompt = self.prompt_mgr.create_prompt(chunk_token_distribution_number=self.chunk_token_distribution_number,
+                                                    chunk_number = self.chunk_number,
+                                                    notation=self.notation,
                                                     generate_title=self.generate_title,
                                                     request_chunks=self.request_chunks,
                                                     token_adjustment=self.token_adjustment,
                                                     prompt_as_previous=self.prompt_as_previous_clone)
+        self.query_js["prompt"]=safe_json_loads(self.prompt)
         self.endpoint = self.model_mgr.selected_endpoint
         self.header = self.api_mgr.header
         self.response = requests.post(url=self.endpoint, json=self.prompt, headers=self.header)
-        self.try_load_response()
-        self.api_response = safe_json_loads(self.get_response())
-        self.query_js={}
-        self.content = get_any_value(self.api_response,'content')
-        if self.content and isinstance(self.content,list):
-            if len(self.content)>0:
-                self.content = safe_json_loads(self.content[0])
-        self.query_js["prompt"]=safe_json_loads(self.prompt)
-        self.query_js["response"] = safe_json_loads(self.api_response)
-        self.query_js["query_response"]=self.content
-        if self.content:
-            for i,key in enumerate(self.response_keys):
-                value = get_any_value(self.content,key)
-                setattr(self,key,value)
-                if key == 'prompt_as_previous' and value:
-                    self.prompt_as_previous_clone=self.api_response
-        self.model= get_any_value(self.query_js["response"],'model') or get_any_value(self.query_js["response"],'error') or 'default' 
-        self.title = self.original_title or self.generate_title or get_any_value(self.query_js["response"],'created')
-        self.save_manager = SaveManager(data=self.query_js,title=self.title,directory=self.directory,model=self.model)
-        self.query_js['file_path']=self.save_manager.file_path
-        self.output.append(self.query_js)
-        
+        safe_json_loads(self.get_response())
     def initial_query(self):
         """
         Manages the initial sequence of sending queries and processing responses.
@@ -207,20 +210,26 @@ class ResponseManager:
         """
         self.query_done=False
         self.i_query = 0
-        for i in range(len(self.token_dist)):
-            response_loop=True
-            abort_it=False
-            while response_loop:
-                self.send_query()
-                if self.get_abort() or self.abort_js["abort"]:
-                    response_loop=False
+        self.total_dists = 0
+        for dist in self.chunk_token_distributions:
+            self.total_dists +=len(dist)
+        for chunk_token_distribution_number,distributions in enumerate(self.chunk_token_distributions):
+            for chunk_number,distribution in enumerate(distributions):
+                self.chunk_token_distribution_number=chunk_token_distribution_number
+                self.chunk_number=chunk_number
+                self.total_current_dist = len(distributions)
+                response_loop=True
+                abort_it=False
+                while response_loop:
+                    self.send_query()
+                    if self.get_abort() or self.abort_js["abort"]:
+                        response_loop=False
+                        break
+                print(f'in while {self.i_query}')
+                if self.abort_js["abort"]:
                     break
-            print(f'in while {i}')
-            if self.abort_js["abort"]:
-                break
-            self.i_query=i
+                self.i_query+=1
+                
         self.query_done=True
         self.i_query=0
         return self.output
-ResponseManager(api_mgr=ApiManager(),
-                prompt_mgr=PromptManager(instruction_mgr=InstructionManager(),model_mgr=ModelManager())).send_query()
