@@ -1,62 +1,47 @@
 # DEPENDENCIES #
 import os
 import pyperclip
-from .. import (
-    ApiManager,
-    ModelManager,
-    PromptManager,
-    ResponseManager,
-    SaveManager,
-    InstructionManager,
-    get_value_from_object,
-    read_from_file_with_multiple_encodings,
-    FileCollator
-)
-from .abstract_ai_gui import get_total_layout
-from .abstract_ai_gui_shared import instructions_keys, all_token_keys, test_options_keys
+from .. import (ApiManager,
+                ModelManager,
+                PromptManager,
+                ResponseManager,
+                SaveManager,
+                InstructionManager,
+                read_from_file_with_multiple_encodings,
+                FileCollator)
+
+from .abstract_ai_gui import (get_total_layout,
+                              instructions_keys,
+                              all_token_keys,
+                              test_options_keys)
+
+from .abstract_gui_navigation import AbstractNavigationManager
+
 from .generate_readme import read_me_window
-from abstract_webtools import UrlManager, SafeRequest, url_grabber_component
-from abstract_gui import (
-    get_event_key_js,
-    text_to_key,
-    AbstractWindowManager,
-    NextReadManager,
-    AbstractBrowser
-)
-from abstract_utilities import (
-    get_any_value,
-    create_new_name,
-    get_sleep,
-    eatAll,
-    safe_json_loads,
-    read_from_file,
-    make_list,
-    ThreadManager,
-    HistoryManager,
-    get_file_create_time,
-    safe_read_from_json
-)
 
+from abstract_webtools import (UrlManager,
+                               SafeRequest,
+                               url_grabber_component)
+
+from abstract_gui import (get_event_key_js,
+                          text_to_key,
+                          AbstractWindowManager,
+                          NextReadManager,
+                          AbstractBrowser)
+
+from abstract_utilities import (get_any_value,
+                                create_new_name,
+                                get_sleep,
+                                eatAll,
+                                safe_json_loads,
+                                read_from_file,
+                                make_list,
+                                ThreadManager,
+                                HistoryManager,
+                                get_file_create_time,
+                                safe_read_from_json,
+                                is_number)
 
-def get_prompt_section_number(event_type,prompt_number,prompt_data):
-    if prompt_number>len(prompt_data)-1:
-        prompt_number = len(prompt_data)-1
-    if event_type == 'back':
-        if prompt_number>0:
-            prompt_number = prompt_number - 1
-    if event_type == 'forward':
-        if prompt_number<len(prompt_data)-1:
-            prompt_number = prompt_number + 1
-    return prompt_number
-def get_prompt_section_sub_number(event_type,prompt_number,prompt_data):
-    if prompt_data:
-        if event_type == 'back':
-            if prompt_number>0:
-                prompt_number = prompt_number - 1
-        if event_type == 'forward':
-            if prompt_number<len(prompt_data)-1:
-                prompt_number = prompt_number + 1
-    return prompt_number
 class GptManager:
     """
     The GptManager class in the abstract_ai module is a complex management system that uses modules to control AI interactions.
@@ -145,40 +130,57 @@ class GptManager:
         AI's responses, the instruction handling, and the submission of queries to the AI. It finally sets a few more internal state variables
         and kicks off the UI's main loop.
  	"""
-        self.window_mgr = AbstractWindowManager()
-        self.window_name = self.window_mgr.add_window(window_name="Chat GPT Console",layout=get_total_layout())
-        self.window_mgr.set_current_window(self.window_name)
-        self.window = self.window_mgr.get_window_method(self.window_name)
-        self.api_call_list=[]
-        self.instruction_bool_keys=[]
+        
         self.values = None
         self.event = None
+        
         self.chunk_title=None
         self.chunk_number=None
-        self.start_query=False
         
+        self.output_list=[]
+        self.latest_output=[]
+        self.prompt_data_list=['']
+        self.request_data_list=['']
+        self.new_response_path_list=[]
+        self.response_directories_list=[]
+        
+        #self.response_path_list = self.aggregate_conversations()
+        
+        self.response=False
+        self.test_bool=False
+        self.loop_one=False
+        self.bool_loop=False
+        self.start_query=False
+        self.updated_progress = False
+        self.api_call_list=[]
+        
+        self.display_number_tracker={'instructions':0,'request':0,'prompt_data':0,'chunk':0,'query':0,'chunk_number':0}
+        self.navigation_mgr= AbstractNavigationManager(self,window_mgr=self.window_mgr,)
         self.browser_mgr = AbstractBrowser(window_mgr=self)
         self.next_read_mgr=NextReadManager()
         self.thread_mgr = ThreadManager()
         self.history_mgr = HistoryManager()
         self.model_mgr = ModelManager()
         self.instruction_mgr = InstructionManager()
-        
-        self.response=False
-        self.updated_progress = False
-        self.test_bool=False
-        self.output_list=[]
-        self.latest_output=[]
-        self.request_data_list=['']
-        self.prompt_data_list=['']
-        self.instructions_list=[{}]
-        self.new_response_path_list=[]
-        self.response_directories_list=[]
-        self.response_path_list = self.aggregate_conversations()
+
         self.initialize_keys()
-        self.loop_one=False
-        self.bool_loop=False
+        
+        self.window_mgr = AbstractWindowManager()
+        self.window_name = self.window_mgr.add_window(window_name="Chat GPT Console",title="Chat GPT Console",layout=get_total_layout())
+        self.window_mgr.set_current_window(self.window_name)
+        self.window = self.window_mgr.get_window_method(self.window_name)
         self.window_mgr.while_window(window_name=self.window_name, event_handlers=self.while_window)
+
+    def unlist_self(self,keys):
+        """
+        A helper method that takes a list of keys as a parameter, iterates over each key, and checks if
+        the value of the corresponding key in the class is a non-empty list. If it is, it updates the class's attribute with the last element in the list.
+ 	"""
+        for key in keys:
+            value = getattr(self,key)
+            if isinstance(value,list) and value:
+                value = value[-1]
+            setattr(self,key,value)
     @staticmethod
     def get_remainder(num):
         return 100-int(num)
@@ -198,11 +200,29 @@ class GptManager:
             for section in sections_list:
                 keys.append(text_to_key(text,section=section))
         return keys
+    @staticmethod
+    def read_file_contents(file_path):
+        """
+        Reads the contents of the file at the given path.
+
+        Args:
+            file_path (str): The path to the file.
+
+        Returns:
+            str: The contents of the file.
+        """
+        if os.path.splitext(file_path)[-1] == '.docx':
+            return read_docx(file_path)
+        else:
+            return read_from_file_with_multiple_encodings(file_path)
+        
     def initialize_keys(self):
         """sets up the necessary key definitions for handling events and values from the GUI. It defines the keys
         needed to check the event, the chunk history, the token percentage dropdowns, the instruction keys, and others.
         This is essential for the proper functioning and interaction of the GptManager with the GUI.
  	"""
+        self.instruction_bool_keys=[]
+        
         self.event_check_keys = ['chunk_event_check',
                                  'response_event_check',
                                  'url_event_check',
@@ -212,21 +232,44 @@ class GptManager:
                                  'model_event_check',
                                  'query_event_check',
                                  'readme_event_check',
-                                 'browser_event_query']
+                                 'browser_event_query',
+                                 'prompt_request_event_check']
+        
+        self.sub_navigation_keys = [text_to_key("chunk back"),
+                                    text_to_key("chunk forward"),
+                                    text_to_key('chunk section back'),
+                                    text_to_key('chunk section forward'),
+                                    text_to_key("query back"),
+                                    text_to_key("query forward"),
+                                    text_to_key('query section back'),
+                                    text_to_key('query section forward'),]
+        
+        self.section_navigation_keys = [text_to_key('instructions section back'),
+                                        text_to_key('instructions section forward'),
+                                        text_to_key('request section back'),
+                                        text_to_key('request section forward'),
+                                        text_to_key('prompt_data section back'),
+                                        text_to_key('prompt_data section forward'),]
+        
+        self.section_navigation_keys = self.section_navigation_keys+self.sub_navigation_keys
+        
         self.chunk_history_name = self.history_mgr.add_history_name('chunk')
+        
         self.toke_percentage_dropdowns = ['-COMPLETION_PERCENTAGE-',
                                           '-PROMPT_PERCENTAGE-']
+        
         self.additions_key_list = self.browser_mgr.key_list+['-FILE_TEXT-',
                                                              '-ADD_FILE_TO_CHUNK-',
                                                              '-ADD_URL_TO_CHUNK-']
+        
         self.instruction_pre_keys = instructions_keys
         for key in self.instruction_pre_keys:
             self.instruction_bool_keys.append(text_to_key(text=key,section='bool'))
+            
         self.sectioned_chunk_text_number_key= text_to_key('chunk text number')
         self.sectioned_chunk_data_key = text_to_key('chunk sectioned data')
-        self.chunk_display_keys=self.get_bool_and_text_keys(
-            all_token_keys
-            )
+        self.chunk_display_keys=self.get_bool_and_text_keys(all_token_keys)
+        
     def update_all(self):
         """
         updates all the needed managers. It updates the Model Manager, Instruction Manager, API Manager,
@@ -239,11 +282,16 @@ class GptManager:
         self.update_response_mgr()
         self.check_test_bool()
 
+    def update_text(self,**kwargs):
+        self.update_instruction_mgr()
+        self.update_prompt_mgr(**kwargs)
+        
     def fill_lists(self):
         if self.prompt_data_list[-1] != '' or self.request_data_list[-1] != '':
             self.prompt_data_list.append('')
             self.request_data_list.append('')
-            self.instructions_list.append({})
+            self.update_text()
+            
 #ApiManagement:
     # Updates the API manager with new values from GUI    
     def update_api_mgr(self):
@@ -251,11 +299,10 @@ class GptManager:
         updates the API manager with new values from the GUI. The values include the API content type,
         header, environment, and key.
  	"""
-        self.content_type=self.window_mgr.get_from_value(text_to_key("content_type"),delim='')
         self.header=self.window_mgr.get_from_value(text_to_key("header"),delim='')
         self.api_env=self.window_mgr.get_from_value(text_to_key("api_env"),delim='')
         self.api_key=self.window_mgr.get_from_value(text_to_key("api_key"),delim='')
-        self.api_mgr = ApiManager(content_type=self.content_type,header=self.header,api_env=self.api_env,api_key=self.api_key)
+        self.api_mgr = ApiManager(header=self.header,api_env=self.api_env,api_key=self.api_key)
 
 #ModelManagement
 # Updates model manager with latest model name from GUI
@@ -277,100 +324,140 @@ class GptManager:
             self.update_model_mgr()
         else:
             return False
-            return True
-    
+        return True
+ 
 #PromptManagement
     # Initializes the prompt manager with the information needed for querying the GPT AI model
-    def update_prompt_mgr(self,prompt_data_list=None,chunk_token_distributions=None,completion_percentage=None,notation=None,chunk_number=None,chunk_type="CODE"):
+    def update_prompt_mgr(self,prompt_data_list=None,request_data_list=None,chunk_token_distributions=None,completion_percentage=None,notation=None,chunk_number=None,chunk_type="CODE"):
         """
         initializes the prompt manager with the information needed for querying the GPT AI model.
 	"""
-        
+        for i in range(len(self.request_data_list)):
+            if self.prompt_data_list[-1] == '' and self.request_data_list[-1] == '':
+                self.prompt_data_list[:-1]
+                self.request_data_list[:-1]
+            else:
+                break
         self.prompt_mgr = PromptManager(instruction_mgr=self.instruction_mgr,
                                    model_mgr=self.model_mgr,
                                    role=self.window_mgr.get_from_value('-ROLE-'),
                                    completion_percentage=completion_percentage or self.window_mgr.get_from_value('-COMPLETION_PERCENTAGE-'),
                                    prompt_data=prompt_data_list or self.prompt_data_list,
-                                   request=self.request_data_list,
+                                   request=request_data_list or self.request_data_list,
                                    notation=self.notation,
                                    chunk_number=chunk_number or self.chunk_number,
                                    chunk_type=chunk_type or self.chunk_type)
         self.chunk_text_number_actual = 0
-        self.window_mgr.update_value(key='-QUERY-',value=self.prompt_mgr.create_prompt(self.chunk_text_number_actual))
-        self.chunk_token_distributions = self.prompt_mgr.chunk_token_distributions
-        if len(self.prompt_data_list) == 0:
-            self.chunk_text_number_display=0
-        else:
-            self.chunk_text_number_display = 1
-        self.window_mgr.update_value(key=text_to_key("chunk number"),value=0)
-        self.window_mgr.update_value(key=text_to_key("chunk section number"),value=0)
-        #self.window_mgr.update_value(key='-CHUNK_TEXT_NUMBER-',value=self.chunk_text_number_display)
-        self.update_chunk_info(0,0)
+        chunk_token_distribution_number = self.get_spec_section_number_display('query')
+        chunk_number = self.display_number_tracker['chunk_number']
+        distributions = self.prompt_mgr.chunk_token_distributions
+        if distributions:
+            if len(distributions)-1<=chunk_token_distribution_number:
+                chunk_token_distribution_number =0
+            chunk_distribution = distributions[chunk_token_distribution_number]
+            if chunk_distribution:
+                if len(chunk_distribution)-1 <= chunk_number:
+                    chunk_number = 0
+            self.chunk_token_distributions=self.prompt_mgr.chunk_token_distributions
+            self.update_chunk_info(chunk_token_distribution_number,chunk_number)
         
     def get_prompt_data_section_number(self):
         return int(self.window_mgr.get_values()[text_to_key("prompt_data section number")])
     
-    def get_chunk_token_distribution_number(self):
-        return int(self.window_mgr.get_values()[text_to_key("chunk section number")])
+    def get_spec_section_number_display(self,variable_header):
+        value = self.display_number_tracker[variable_header]
+        if not is_number(value):
+            value = int(self.display_number_tracker[variable_header])
+            self.window_mgr.update_value(key=text_to_key(f"{variable_header} section number"),value=value)
+        return int(value)
     
-    def get_chunk_number(self):
-        return int(self.window_mgr.get_values()[text_to_key("chunk number")])
+    def update_request_data_list(self,data=None):
+        spec_section_number = self.get_spec_section_number_display(variable_header='request')
+        if data == None:
+            self.request_data_list[spec_section_number] = self.window_mgr.get_values()[text_to_key('request')]
+        else:
+            self.request_data_list[spec_section_number] = data
+            self.update_request_data_display()
+            
+    def update_request_data_display(self,section_number=None):
+        if section_number == None:
+            section_number = self.get_spec_section_number_display(variable_header='request')
+        self.window_mgr.update_value(key=text_to_key('request'),value=self.request_data_list[section_number])
+
+    def update_prompt_data_list(self,data=None):
+        spec_section_number = self.get_spec_section_number_display(variable_header='prompt_data')
+        if data == None:
+            self.prompt_data_list[spec_section_number] = self.window_mgr.get_values()[text_to_key('prompt_data data')]
+        else:
+            self.prompt_data_list[spec_section_number] = data
+            self.update_prompt_data_display()
+            
+    def update_prompt_data_display(self,section_number=None):
+        if section_number == None:
+            section_number = self.get_spec_section_number_display(variable_header='prompt_data')
+        self.window_mgr.update_value(key=text_to_key('prompt_data data'),value=self.prompt_data_list[section_number])
+        
+    def prompt_request_event_check(self):
+        if self.event == text_to_key('request'):
+            self.update_request_data_list()
+            
+        elif self.event == text_to_key('prompt_data data'):
+            self.update_prompt_data_list()
+            
+        elif self.event == '-ADD_QUERY-':
+            pass
+        else:
+            return False
+        self.fill_lists()
+        return True
     
-    def update_prompt_data(data):
+    def update_prompt_data(self,data):
         self.history_mgr.add_to_history(name=self.chunk_history_name,data=self.prompt_data_list)
-        self.prompt_data_list[self.get_prompt_data_section_number()] = data
-        self.window_mgr.update_value(key='-PROMPT_DATA_DATA-',value=data)
+        self.update_prompt_data_list(data=data)
         return self.prompt_data_list
 
-    def update_request_data(data):
+    def update_request_data(self,data):
         self.history_mgr.add_to_history(name=self.request_history_name,data=self.request_data_list)
-        self.request_data_list[self.get_prompt_data_section_number()] = data
-        self.window_mgr.update_value(key='-REQUEST-',value=data)
+        self.update_request_data_list(data=data)
         return self.request_data_list
     
-    def update_chunk_info(self,chunk_iteration=0,chunk_number=0):
-        if self.chunk_token_distributions:
-            if chunk_iteration < len(self.chunk_token_distributions) and chunk_iteration >=0:
-                self.chunk_token_distribution = self.chunk_token_distributions[int(chunk_iteration)][int(chunk_number)]
+    def update_chunk_info(self,chunk_token_distribution_number=0,chunk_number=0):
+        if self.prompt_mgr.chunk_token_distributions:
+            chunk_token_distribution_number=int(chunk_token_distribution_number)
+            if int(chunk_token_distribution_number) < len(self.prompt_mgr.chunk_token_distributions) and chunk_token_distribution_number >=0:
+                self.chunk_token_distribution = self.chunk_token_distributions[int(chunk_token_distribution_number)][int(chunk_number)]
                 self.window_mgr.update_value(key=self.sectioned_chunk_data_key, value=self.chunk_token_distribution['chunk']['data'])
                 for key in self.chunk_display_keys:
                     spl = key[1:-1].lower().split('_')
                     if spl[0] in self.chunk_token_distribution:
                         if spl[-1] in self.chunk_token_distribution[spl[0]]:
                             self.window_mgr.update_value(key=key,value=self.chunk_token_distribution[spl[0]][spl[-1]])
-                            
-    def adjust_chunk_display(self,num):
+            
+    def adjust_chunk_display(self,event_type,chunk_token_distribution_number,chunk_number=0):
         """
         manages the GUI's chunk navigation buttons 'Back' and 'Forward'.
         It ensures that the navigation is within the limits of the total chunks and updates the GUI with the correct chunk.
  	"""
-        self.chunk_text_number_actual+=num
-        self.chunk_text_number_display+=num
-        self.window_mgr.update_value(key='-QUERY-',value=self.prompt_mgr.create_prompt(self.chunk_text_number_actual))
-        self.update_chunk_info(chunk_iteration=self.chunk_text_number_actual)
         
-        self.window_mgr.update_value(key='-CHUNK_TEXT_NUMBER-',value=self.chunk_text_number_display)        
-    def get_chunk_display_numbers(self):
-        """
-        This helper method retrieves the number of chunks that need to be displayed on the GUI.
- 	"""
-        self.chunk_text_number_display = int(self.window_mgr.get_from_value('-CHUNK_TEXT_NUMBER-'))
-        if self.chunk_text_number_display >0:
-            self.chunk_text_number_actual = self.chunk_text_number_display-1
-            
-    def determine_chunk_display(self,event):
-        """
-        determines the chunk to be displayed based on the chunk navigation buttons 'Back' or 'Forward'.
- 	"""
-        self.get_chunk_display_numbers()
-        if len(self.prompt_data)>0:
-            if self.event == '-CHUNK_TEXT_BACK-':
-                if self.chunk_text_number_actual >0:
-                    self.adjust_chunk_display(-1)
-            elif self.event == '-CHUNK_TEXT_FORWARD-':
-                if self.chunk_text_number_display < len(self.token_dist):
-                    self.adjust_chunk_display(1)
-                    
+        #update chunk data
+        self.window_mgr.update_value(text_to_key('chunk sectioned data'),
+                                     self.chunk_token_distributions[chunk_token_distribution_number][chunk_number]['chunk']['data'])
+        #get query
+        self.window_mgr.update_value(key='-QUERY-',
+                                     value=self.prompt_mgr.create_prompt(chunk_token_distribution_number=chunk_token_distribution_number,chunk_number=chunk_number))
+        #update chunk info
+        self.update_chunk_info(chunk_token_distribution_number,
+                               chunk_number)
+        #change display numbers
+        self.window_mgr.update_value(text_to_key("chunk section number"),chunk_token_distribution_number)
+        self.window_mgr.update_value(text_to_key("query section number"),chunk_token_distribution_number)
+        self.window_mgr.update_value(text_to_key("chunk number"),chunk_number)
+        self.window_mgr.update_value(text_to_key("query number"),chunk_number)
+
+        #update chunk keys
+        self.update_chunk_info(chunk_token_distribution_number=chunk_token_distribution_number,chunk_number=chunk_number)
+        return chunk_token_distribution_number
+         
     def add_to_chunk(self,content):
         """
         adds the user's entered request to the request chunk. It also updates the 'chunk_text_number' in the GUI.
@@ -378,20 +465,25 @@ class GptManager:
         if self.window_mgr.get_from_value('-AUTO_CHUNK_TITLE-'):
             if self.chunk_title:
                 content="# SOURCE #\n"+self.chunk_title+'\n# CONTENT #\n'+content+"\n# END SOURCE #"+self.chunk_title
-        if self.prompt_data_list[self.get_prompt_data_section_number()]:
-            content = self.prompt_data_list[self.get_prompt_data_section_number()]+'\n\n'+content
+        prompt_data = self.prompt_data_list[self.get_prompt_data_section_number()]
+        if prompt_data:
+            content = prompt_data+'\n\n'+content
         content = eatAll(content,['\n'])
         return content
-
-        
+    
     ## chunk data keys
     def chunk_event_check(self):
         """
         checks if any events related to the chunk UI elements were triggered and performs the necessary actions.
  	"""
-        if self.event in ['-CHUNK_TEXT_BACK-','-CHUNK_TEXT_FORWARD-']:
-            self.determine_chunk_display(self.event)
-            
+        
+        if self.event in self.section_navigation_keys:
+            nav_type, nav_direction=self.navigation_mgr.parse_navigation_event(self.event)
+            nav_data=self.navigation_mgr.get_navigation_data(nav_type)
+            sub_section_number = self.navigation_mgr.get_sub_section_number(nav_type)
+            refference_object = self.navigation_mgr.get_reference_object(nav_type)
+            self.navigation_mgr.update_navigation_counters(nav_data, nav_direction)
+
         elif self.event in ['-REDO_CHUNKS-',
                             '-ADD_URL_TO_CHUNK-',
                             '-ADD_FILE_TO_CHUNK-',
@@ -400,7 +492,6 @@ class GptManager:
                                                                                         '-ADD_FILE_TO_CHUNK-',
                                                                                         '-ADD_URL_TO_CHUNK-']:
             data=None
-            completion_percentage=None
             if self.event == '-CLEAR_CHUNKS-':
                 """
                 clears the currently displayed chunk data from the GUI.
@@ -432,12 +523,11 @@ class GptManager:
                 self.chunk_title=self.window_mgr.get_values()[text_to_key('-CHUNK_TITLE-',section='files')]
                 data = self.add_to_chunk(self.window_mgr.get_values()[self.script_event_js['-FILE_TEXT-']])
                 self.chunk_type='CODE'
-                
-            self.update_prompt_mgr(prompt_data_list=self.update_prompt_data(data),completion_percentage=completion_percentage)
+            self.update_prompt_data(data)
         else:
             return False
         return True
-    
+        
 #InstructionManagement
     # updates instructions manager with environment values from GUI      
     def update_instruction_mgr(self):
@@ -451,12 +541,12 @@ class GptManager:
                 value = self.window_mgr.get_from_value(text_to_key(text=key,section="TEXT"))
                 if value == '':
                     value = True
-            setattr(self, self.instruction_pre_keys[i], value)
+            setattr(self, key, value)
         self.api_response=True
         if not self.window_mgr.get_from_value(text_to_key(text='instructions',section="BOOL")):
             self.api_response=False
             for i,key in enumerate(self.instruction_pre_keys):
-                setattr(self, self.instruction_pre_keys[i], None)
+                setattr(self, key, None)
         self.instruction_mgr = InstructionManager(api_response=self.api_response,
                                                   notation=self.notation,
                                                   suggestions=self.suggestions,
@@ -501,12 +591,12 @@ class GptManager:
         It then updates the Prompt Manager with the latest instructions.
  	"""
         if self.event in self.instruction_bool_keys:
-            
-            self.update_instruction_mgr()
-            self.update_prompt_mgr()
+            self.update_text()
+        else:
+            return False
+        return True
             
 #urlManagement
-        
     def append_output(self,key,new_content):
         """
         appends a line of text to the 'other' section of feedback on the GUI.
@@ -592,6 +682,7 @@ class GptManager:
         call_name = create_new_name(name='api_call',names_list=self.api_call_list)
         if call_name not in self.api_call_list:
             self.api_call_list.append(call_name)
+    
     def query_event_check(self):
         """
         checks if the '-SUBMIT_QUERY-' event was triggered. If it was, it creates a new API call name, initializes
@@ -640,17 +731,19 @@ class GptManager:
         self.total_dists=0
         for dist in self.chunk_token_distributions:
             self.total_dists +=len(dist)
-        chunk = self.total_dists
         i_query = int(self.response_mgr.i_query)
         if done == True:
             self.window['-PROGRESS-'].update_bar(100, 100)
-            self.window_mgr.update_value(key='-QUERY_COUNT-', value=f"a total of {chunk} chunks have been sent")
+            self.window_mgr.update_value(key='-QUERY_COUNT-', value=f"a total of {self.total_dists} chunks have been sent")
             self.window_mgr.update_value(key='-PROGRESS_TEXT-', value='SENT')
             self.updated_progress = True
         else:
             self.get_dots()
-            self.window['-PROGRESS-'].update_bar(min(i_query,1), min(chunk,2))
-            self.window_mgr.update_value(key='-QUERY_COUNT-', value=f"chunk {i_query+1} of {max(chunk,1)} being sent")
+            self.window['-PROGRESS-'].update_bar(i_query, self.total_dists)
+            if i_query == 0 and self.total_dists!= 0 :
+                i_query = 1
+            self.window_mgr.update_value(key='-QUERY_COUNT-', value=f"chunk {i_query} of {self.total_dists} being sent")
+            
     # checks if the Response Manager has completed the query process.         
     def check_response_mgr_status(self):
         """
@@ -784,16 +877,7 @@ class GptManager:
                     return len(request_data_list)-1
                 return i
         return len(self.request_data_list)-1
-    def unlist_self(self,keys):
-        """
-        A helper method that takes a list of keys as a parameter, iterates over each key, and checks if
-        the value of the corresponding key in the class is a non-empty list. If it is, it updates the class's attribute with the last element in the list.
- 	"""
-        for key in keys:
-            value = getattr(self,key)
-            if isinstance(value,list) and value:
-                value = value[-1]
-            setattr(self,key,value)
+
     def initialize_output_display(self):
         """
         Initializes the display of the latest output in the GUI.
@@ -850,47 +934,41 @@ class GptManager:
         Returns:
             bool: True if a listbox event has been triggered, otherwise False
         """
-        event_triggered = False
-        values = self.window_mgr.get_values()
-        file_path = None
-
-        if self.script_event_js['found'] == '-FILES_LIST-':
-            selected_files = values[self.event]
-            if selected_files:
-                file_path = selected_files[0]
-                event_triggered = True
-        elif self.script_event_js['found'] == '-BROWSER_LIST-':
-            directory = values[self.script_event_js['-DIR-']]
-            selected_files = values[self.script_event_js['-BROWSER_LIST-']]
-            if selected_files:
-                file_path = os.path.join(directory, selected_files[0])
-                event_triggered = True
-
-        if file_path and os.path.isfile(file_path):
+        def open_folder():
+            file_path = self.window_mgr.get_values()[self.script_event_js['-DIR-']]
+            files_list = self.window_mgr.get_values()[self.script_event_js['-BROWSER_LIST-']]
+            if file_path:
+                file_path = os.path.join(file_path,files_list[0])
+                if not os.path.isfile(file_path):
+                    self.browser_mgr.handle_event(self.window_mgr.get_values(),self.event,self.window)
+        def open_file(file_path):
             try:
-                contents = read_file_contents(file_path)  # Assuming read_file_contents is defined to handle different file types
+                contents = self.read_file_contents(file_path)  # Assuming read_file_contents is defined to handle different file types
                 self.window_mgr.update_value(key=self.script_event_js['-FILE_TEXT-'], value=safe_json_loads(contents))
                 self.chunk_title = os.path.basename(file_path)
                 self.window_mgr.update_value(key=text_to_key('-CHUNK_TITLE-', section='files'), value=self.chunk_title)
             except Exception as e:
                 print(f"Cannot read from path {file_path}. Error: {e}")
+        event_triggered = False
+        file_path = None
+        if self.script_event_js['found'] == '-FILES_LIST-':
+            selected_files = self.values[self.script_event_js['-FILES_LIST-']]
+            if selected_files:
+                file_path = selected_files[0]
+                event_triggered = True
+        elif self.script_event_js['found'] == '-BROWSER_LIST-':
+            directory = self.values[self.script_event_js['-DIR-']]
+            selected_files = self.values[self.script_event_js['-BROWSER_LIST-']]
+            if selected_files:
+                file_path = os.path.join(directory, selected_files[0])
+                event_triggered = True
+        if file_path:
+            if os.path.isfile(file_path):
+                open_file(file_path)
+            else:
+                open_folder()
         return event_triggered
-    def read_file_contents(file_path):
-        """
-        Reads the contents of the file at the given path.
-
-        Args:
-            file_path (str): The path to the file.
-
-        Returns:
-            str: The contents of the file.
-        """
-        if os.path.splitext(file_path)[-1] == '.docx':
-            return read_docx(file_path)
-        else:
-            return read_from_file_with_multiple_encodings(file_path)
-            
-
+    
     def update_last_response_file(self):
         """
         Updates the path to the last file that recorded a response.
@@ -1017,58 +1095,7 @@ class GptManager:
             # If you want to stop checking once bool_loop is True (or some condition), you can break here
             if self.bool_loop:  # Assuming self.bool_loop should stop the loop when True
                 break
-        prompt_number = self.get_prompt_data_section_number()
-        
-        if self.event == text_to_key('prompt_data data'):
-            self.prompt_data_list[prompt_number] = self.window_mgr.get_values()[text_to_key('prompt_data data')]
-            
-        if self.event == text_to_key('request'):
-            self.request_data_list[prompt_number] = self.window_mgr.get_values()[text_to_key('request')]
-            
-        elif self.event == '-ADD_QUERY-':
-            self.fill_lists()
-            
-        elif self.event in [text_to_key('instructions section back'),
-                          text_to_key('instructions section forward'),
-                          text_to_key('request section back'),
-                          text_to_key('request section forward'),
-                          text_to_key('prompt_data section back'),
-                          text_to_key('prompt_data section forward')]:
-            
-            prompt_number=get_prompt_section_number(self.event.split('_')[-1][:-1].lower(),prompt_number,self.prompt_data_list)
-            for header in ['instructions','request','prompt_data']:
-                self.window_mgr.update_value(text_to_key(f"{header} section number"),prompt_number)
-            self.update_bool_text_instructions()
-            self.window_mgr.update_value(text_to_key('prompt_data data'),self.prompt_data_list[prompt_number])
-            self.window_mgr.update_value(text_to_key('request'),self.request_data_list[prompt_number])
-            
-        elif self.event in [text_to_key("chunk section forward"),
-                          text_to_key("chunk section back"),
-                          text_to_key("query section forward"),
-                          text_to_key("query section back"),
-                          text_to_key("chunk forward"),
-                          text_to_key("chunk back"),
-                          text_to_key("query forward"),
-                          text_to_key("query back")]:
-            
-            chunk_token_distribution_number = self.get_chunk_token_distribution_number()
-            chunk_number = 0
-            if "SECTION" in self.event:
-                chunk_token_distribution_number=get_prompt_section_number(self.event.split('_')[-1][:-1].lower(),chunk_token_distribution_number,self.chunk_token_distributions)
-                self.window_mgr.update_value(text_to_key("chunk section number"),chunk_token_distribution_number)
-                self.window_mgr.update_value(text_to_key("query section number"),chunk_token_distribution_number)
-                
-            else:
-                chunk_number = self.get_chunk_number()
-                chunk_number= get_prompt_section_number(self.event.split('_')[-1][:-1].lower(),chunk_number,self.chunk_token_distributions[chunk_token_distribution_number])            
-                self.window_mgr.update_value(text_to_key("chunk number"),chunk_number)
-                self.window_mgr.update_value(text_to_key("query number"),chunk_number)
-            self.window_mgr.update_value(text_to_key('chunk sectioned data'),self.chunk_token_distributions[chunk_token_distribution_number][chunk_number]['chunk']['data'])
-            self.window_mgr.update_value(key='-QUERY-',value=self.prompt_mgr.create_prompt(chunk_token_distribution_number=chunk_token_distribution_number,chunk_number=chunk_number))
-            self.update_chunk_info(chunk_token_distribution_number,chunk_number)
-        
-        self.loop_one=True
-        
+
     def browser_event_query(self,values=None,event=None,window=None):
         values=values or self.window_mgr.get_values()
         event=event or self.window_mgr.get_event()
@@ -1082,10 +1109,13 @@ class GptManager:
             for browser_event in ['-SCAN_FILES-',
                                   '-SCAN_RESPONSES-']:
                 self.browser_event_query(self.values,browser_event,self.window)
-        self.next_read_mgr.execute_queue()
-        self.script_event_js = get_event_key_js(event = self.event,key_list=self.additions_key_list)
-        self.perform_event_checks()
-        
+        try:
+            self.next_read_mgr.execute_queue()
+            self.script_event_js = get_event_key_js(event = self.event,key_list=self.additions_key_list)
+            self.perform_event_checks()
+            self.loop_one=True
+        except:
+            pass
 #TestManagement
     def check_test_bool(self):
         """
@@ -1121,3 +1151,4 @@ class GptManager:
         else:
             return False
         return True
+
